/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */

import javax.swing.table.AbstractTableModel;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * A TableModel implementation used by the s2iConfigureNodes class. It parses
 * a Spice circuit to determine the top-level subckt terminals that are
 * unconnected, and must be configured.
 *
 * * @author Anish Malhotra, Cadence Design Systems
 */

public class s2iConfigTableModel extends AbstractTableModel{

private static final int FIFTY = 50;
private static final int HUNDRED = 100;
private static Object[][] data = new Object[FIFTY][];
private static Object[][] data2 = new Object[FIFTY][];  //Used for mark & cancel
private static Hashtable elemNames = new Hashtable();
private static boolean marked = false;
private static String[] colNames;
private static int rowCtr = 0;
private static int data2RowCtr = 0;
private static final int maxCol = 6;  // 0 to 5 = 6


/**
 * Get the number of columns in the table (fixed at 6).
 *
 * @author Anish Malhotra, Cadence Design Systems
 * @return The number of columns.
 */

public int getColumnCount()
{
   return maxCol;
}

/**
 * Get the number of rows in the table, equal to the number of top-level
 * unconnected subckt terminals.
 *
 * @return The number of rows.
 */

public int getRowCount()
{
   return rowCtr;
}

/**
 * Get the name of a column by position. The first column is at position 0.
 *
 * @param col The position of the column.
 * @return The name String of the column.
 */

public String getColumnName(int col)
{
   //return colNames[col];
  String name;

  switch(col){
    case 0  : name = new String("Net/Node");
              break;
    case 1  : name = new String("Pin Type");
              break;
    case 2  : name = new String("  R  ");
              break;
    case 3  : name = new String("Typ. voltage");
              break;
    case 4  : name = new String("Min. voltage");
              break;
    case 5  : name = new String("Max. voltage");
              break;
    default : name = new String("Column " + (new Integer(col-1)).toString());
              break;
  }
  return name;
}

/**
 * Get the value of a table cell by position.
 *
 * @param row The row position of the cell.
 * @param col The column position of the cell.
 * @return The object at the cell location, or null for invalid/empty location.
 */

public Object getValueAt(int row, int col)
{
  if (isCellValid(row, col))
    return data[row][col];

  return null;
}

/**
 * Get the name of a node by row position. The first column is the name.
 *
 * @param row The row position of the node.
 * @return The name String of the node, or null for invalid/empty location.
 */

public String getNode(int row)
{
  if (row<rowCtr)
    return( (String) getValueAt(row, 0));

  return null;
}

/**
 * Get the pin type of a node by row position. The 2nd column is the pin type.
 *
 * @param row The row position of the node.
 * @return The pin type String of the node, or null for invalid/empty location.
 */

public String getPinType(int row)
{
  if (row<rowCtr)
    return( (String) getValueAt(row, 1));

  return null;
}

/**
 * Get the termination resistance value of a Float node by row position. The
 * 3rd column is the termination resistance.
 *
 * @param row The row position of the node.
 * @return The termination resistance String of the node, or null for invalid/empty location.
 */

public String getResistance(int row)
{
  if (row<rowCtr)
    return( (String) getValueAt(row, 2));

  return null;
}

/**
 * Get the typical mode termination voltage value of a Float node by row
 * position. The 4th column is the typical mode termination voltage.
 *
 * @param row The row position of the node.
 * @return The termination voltage String of the node, or null for invalid/empty location.
 */

public String getTypVoltage(int row)
{
  if (row<rowCtr)
    return( (String) getValueAt(row, 3));

  return null;
}

/**
 * Get the minimum mode termination voltage value of a Float node by row
 * position. The 5th column is the minimum mode termination voltage.
 *
 * @param row The row position of the node.
 * @return The termination voltage String of the node, or null for invalid/empty location.
 */

public String getMinVoltage(int row)
{
  if (row<rowCtr)
    return( (String) getValueAt(row, 4));

  return null;
}

/**
 * Get the maximum mode termination voltage value of a Float node by row
 * position. The 5th column is the maximum mode termination voltage.
 *
 * @param row The row position of the node.
 * @return The termination voltage String of the node, or null for invalid/empty location.
 */

public String getMaxVoltage(int row)
{
  if (row<rowCtr)
    return( (String) getValueAt(row, 5));

  return null;
}

/**
 * Determine if the data at a cell location is valid. Valid only means that
 * there CAN be a value at that location.
 *
 * @param row The row position of the node.
 * @return true if the cell is valid.
 */

private boolean isCellValid(int row, int col)
{
   Object value = data[row][1];
   if ((row<rowCtr) && (col < 2))
      return true;  //First column(Node/Net name is always valid

   if (value != null)
   {
      String pinType = (String) value;
      if (pinType.trim().compareTo("Float") == 0)
         return (true);
      else //Only the pin-type is editable
        if (col == 1)
           return(true);
   }

   return(false);
}


/**
 * The core data of the tree is stored in the 2D array of objects. This
 * data is static. So any subsequent invocation will already contain all
 * the previous values. But, the following still needs to be done to do
 * a proper sync up:
 *  - add the new nodes.
 *  - delete the missing ones.
 *  - preserve, along with the settings, the existing ones.
 *
 * @param elem The Enumeration of elements to update the table for.
 */

public void updateData(java.util.Enumeration elem)
{
   //Put the enumeration entries in a Hash. Also add the new ones.
   java.util.Hashtable elemHash = new Hashtable();
   String element;
   while (elem.hasMoreElements())
   {
      element = (String) elem.nextElement();
      addInstance(element);
      elemHash.put(element, element);
   }

   //delete the missing ones
   String elemName;
   int shift = 0;
   for(int row = 0; row < rowCtr; row ++)
   {
      elemName = getNode(row);
      if (shift > 0)
      {
         data[row-shift] = data[row];
      }
      if (!(elemHash.containsKey(elemName)))
      {
         shift++;
         elemNames.remove(elemName);
      }
   }
   rowCtr -= shift;

}


/**
 * Add a row to the table data by parsing the row data in String form.
 *
 * @param entry The row data in String form.
 * @return true if the cell is valid.
 */

public void addInstance(String entry)
{
   if (!elemNames.containsKey(entry))
   {
      int colCtr = 0;
      String netName = null;
      Object[] Row = new Object[maxCol];

      if (entry == null) return;
      StringTokenizer st = new StringTokenizer(entry);
      String Token = null;
      if (st.hasMoreTokens())
      {
         Token = st.nextToken();
         netName = Token;
         Row[colCtr++] = Token;
      }
      while (st.hasMoreTokens())
      {
         Token = st.nextToken();
         if (Token.equals("null"))
             Token = " ";
         Row[colCtr++] = Token;
      }

      // If the Definition had at least one token then increment the rows in data
      if (netName!= null)
      {
         data[rowCtr++] = Row;
         elemNames.put(netName, netName);
      }
   }
}

/**
 * Print the table data to stdout. This is for debugging.
 */

public void printdataNcol()
{
   int row, col;
   row =0; col = 0;
   String dbg = "";

   // Print the column Names
   while (col < maxCol)
      dbg = dbg + "\t" + getColumnName(col++);
   dbg = dbg + "\n";

   // Print the data
   while (row < rowCtr)
   {
      col = 0;
      dbg = dbg + "\nRow " + row +": ";
      while (col < maxCol)
         dbg = dbg + "\t" + data[row][col++];
      dbg = dbg + "\n";
      row++;
   }
   s2iError.Debug(dbg);
}

/**
 * Convert the table data to a vector of row Strings, ready for storing.
 *
 * @return The Vector of Strings.
 */

public java.util.Vector archive()
{
   StringBuffer Row = new StringBuffer();
   java.util.Vector tmpVector = new java.util.Vector();

   for (int row =0; row < rowCtr ; row++)
   {
      Row.setLength(0);
      for (int col =0; col < maxCol; col++)
      {
         Row.append( ((String) data[row][col]) + " ");
      }
      tmpVector.addElement(Row.toString());
   }

   return(tmpVector);
}

/**
 * Copy the table data to a buffer, used for restoring later.
 */

public void mark()
{
   Object[] Row;

   //Copy data to data2
   for (int row =0; row < rowCtr ; row++)
   {
      Row = new Object[maxCol];
      for (int col =0; col < maxCol; col++)
      {
         Row[col] = data[row][col];
      }
      data2[row] = Row;
   }
   data2RowCtr = rowCtr;

   marked = true;
}

/**
 * Restore the table data from the mark buffer.
 */

public void restore()
{
   if (marked)
   {
      Object[] Row;
      //Copy data2 to data
      for (int row =0; row < data2RowCtr ; row++)
      {
         Row = new Object[maxCol];
         for (int col =0; col < maxCol; col++)
         {
            Row[col] = data2[row][col];
         }
         data[row] = Row;
      }
      rowCtr = data2RowCtr;
   }
}
  

/**
 * Determine if a cell can be edited by the user.
 *
 * @param row The cell row number.
 * @param col The cell column number.
 * @return true if the cell is editable.
 */
   public boolean isCellEditable(int row, int col)
   {
     //Note that the data/cell address is constant,
     //no matter where the cell appears onscreen.
     return (col != 0 && isCellValid(row, col));
   }


/**
 * Set a cell value.
 *
 * @param value The value to store into the cell.
 * @param row The cell row number.
 * @param col The cell column number.
 */
 public void setValueAt(Object value, int row, int col)
 {
    data[row][col] = value;
 }


/**
 * Main routine for testing.
 * This will add a few dummy circuits and then print out data and colNames
 *
 @param arg unused
 */
public static void main(String[] arg)
{

  s2iConfigTableModel configurationTableModel = new s2iConfigTableModel();

  configurationTableModel.addInstance("CKT1 CKT1_1 VCC_1 GND_1 PIN_1 POUT_1");
  configurationTableModel.addInstance("CKT2 CKT2_2 N1_2 N2_2 N3_2 N4_2 N5_2");
  configurationTableModel.addInstance("CKT3 Node1_3 Node2_3");

  configurationTableModel.printdataNcol();
  return ;
}

}
