/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */

import javax.swing.table.AbstractTableModel;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * A TableModel class to keep the list of instantiated top-level subckts.
 *
 * @author Anish Malhotra, Cadence Design Systems
 */
public class s2iInstCktTableModel extends AbstractTableModel{

/**
 * Number of instance rows to allocate space for.
 */
private static final int rowLimit = 50;
/**
 * Number of instance columns (elem + def + nodes) to allocate space for.
 */
private static final int colLimit = 100;
/**
 * The instance data.
 */
protected Object[][] data = new Object[50][];
/**
 * Number of rows currently in use.
 */
protected int rowCtr = 0;
/**
 * Number of columns currently in use (widest row).
 */
protected int maxCol = 0;

/**
 * Clear all data from the table.
 */
public void clearAll()
{
   data = new Object[50][];
   rowCtr = 0;
   maxCol = 0;
}

/**
 * Return the number of columns in the widest row.
 * @return Number of columns
 */
public int getColumnCount()
{
   return maxCol;
}

/**
 * Return the number of rows defined.
 * @return Number of rows
 */
public int getRowCount()
{
   return rowCtr;
}

/**
 * Get the name of a column by position
 * @param column number (first column is 0)
 * @return Name of column
 */
public String getColumnName(int col)
{
  String name;

  switch(col){
    case 0  : name = new String("SUBCKT");
              break;
    case 1  : name = new String("NAME");
              break;
    default : name = new String("Node " + (new Integer(col-1)).toString());
              break;
  }
  return name;
}

/**
 * Return the value of a cell by position.
 * @param row The cell row
 * @param col The cell column
 * @return Cell contents (may be null)
 */
public Object getValueAt(int row, int col)
{
   return data[row][col];
}

/**
 * Determine if the value of a cell is null
 * @param row The cell row
 * @param col The cell column
 * @return true if cell value is null)
 */
public boolean isCellBlank(int row, int col)
{
   return( data[row][col] == null);
}

/**
 * Add a new row for an instance. The instance is given as a String of words.
 * @param CktDef The String of instance tokens
 */
public void addInstance(String CktDef)
{
   int colCtr = 0;
   String cktName = null;
   Object[] Row = new Object[100];

   if (CktDef == null) return;
   StringTokenizer st = new StringTokenizer(CktDef);
   String Token = null;
   if (st.hasMoreTokens())
   {
      Token = st.nextToken();
      cktName = Token;
      Row[colCtr++] = Token;
   }
   while (st.hasMoreTokens())
   {
      Token = st.nextToken();
      Row[colCtr++] = Token;
   }
   if (maxCol < colCtr) maxCol = colCtr;

   // If the Definition had at least one token then increment the rows in data
   if (cktName!= null)
      data[rowCtr++] = Row;
}

/**
 * Print the table data to stdout in DEBUG mode.
 */
public void printdataNcol()
{
   int row, col;
   row =0; col = 0;
   String dbg = "";

   // Print the column Names
   while (col < maxCol)
      dbg = dbg + "\t" + getColumnName(col++);
   dbg = dbg + "\n";

   // Print the data
   while (row < rowCtr)
   {
      col = 0;
      dbg = dbg + "\nRow " + row +": ";
      while (col < maxCol)
         dbg = dbg + "\t" + data[row][col++];
      dbg = dbg + "\n";
      row++;
   }
   s2iError.Debug(dbg);
}

/**
 * Connvert the table data to a Vector of Strings, which can be stored.
 * @return The Vector of instance token Strings
 */
public java.util.Vector archive()
{
   int row, col;
   row =0; col = 0;
   StringBuffer tmpStr = new StringBuffer();
   java.util.Vector tmpVector = new java.util.Vector();

   // Print the data
   while (row < rowCtr)
   {
      col = 0;
      tmpStr.setLength(0);
      while (col < maxCol)
      {
         if (data[row][col] != null)
            tmpStr.append( ((String) data[row][col]) + " " );
         col++;
      }
      row++;
      tmpVector.addElement(tmpStr.toString());
   }

   return(tmpVector);
}

/**
 * Get an enumeration of unique node names present in the data model.
 * @return Enumeration of node name Strings
 */
 public Enumeration getNodes()
 {
    Hashtable nodes = new Hashtable();
    Object value;
    String nodeName;
    for (int i =0; i<rowCtr; i++)
    {
       for (int j=2; j<maxCol; j++)
       {
          value= data[i][j];
          if (value != null)
          {
             nodeName = (String) value;
             nodes.put(nodeName, nodeName);
          }
          else
            break;
       }
    }

    return (nodes.elements());
 }

/**
 * Determine if the user can edit a cell value.
 * @param row The cell row
 * @param col The cell column
 * @return true if the cell can be edited
 */
 public boolean isCellEditable(int row, int col)
 {
   if (col == 0)
     return false;  // Only the sub-circuit is non-editable

   return (!isCellBlank(row, col));
 }

/**
 * Set a cell value.
 * @param value The value to set the cell to
 * @param row The cell row
 * @param col The cell column
 */
 public void setValueAt(Object value, int row, int col)
 {
    boolean ValueBlank = (((String) value).trim().compareTo("") == 0);
    boolean fieldBlank = isCellBlank(row, col);
    if ((!fieldBlank) && ValueBlank)
    {
       String Msg = "You must type a name.";
       s2iError.PrintMsg(Msg, s2iError.ERR, s2iError.POPUP, null);
    }
    else if ( fieldBlank && (!ValueBlank))
    {
       String Msg = "Number of Arguments does not match the definition.";
       s2iError.PrintMsg(Msg, s2iError.ERR, s2iError.POPUP, null);
    }
    else
       data[row][col] = value;
 }

/**
 * A main method for testing this class.
 * This will add a few dummy circuits and then print out data and col Names
 * @param args unused
 */
public static void main(String[] args)
{

  s2iInstCktTableModel s2iInstCktTableModel = new s2iInstCktTableModel();

  s2iInstCktTableModel.addInstance("CKT1 CKT1_1 VCC_1 GND_1 PIN_1 POUT_1");
  s2iInstCktTableModel.addInstance("CKT2 CKT2_2 N1_2 N2_2 N3_2 N4_2 N5_2");
  s2iInstCktTableModel.addInstance("CKT3 Node1_3 Node2_3");

  s2iInstCktTableModel.printdataNcol();
  return ;
}

}
